//******************************************************************************//
//
//  Sample application illustrating the usage of the CSQLiteDatabase and
//  CSQLiteRecordset classes, used to access SQLite databases from MFC
//  applications
//
//  Designed for the Oxetta report builder, the free report design and print
//  library for C/C++ developers
//
//  http://www.oxetta.com for more details
//
//******************************************************************************//

#include "stdafx.h"
#include "Sample.h"
#include "SampleDlg.h"
#include "afxdialogex.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CSampleDlg::CSampleDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CSampleDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_pDatabase = NULL;
}

void CSampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_EMPLOYEE, m_lstEmployee);

	if (!pDX->m_bSaveAndValidate)
	{
	//  creating the database object
		m_pDatabase = new CSQLiteDatabase;

	//  constructing the path to the database file
		TCHAR szPath[MAX_PATH + 1];
		GetModuleFileName(GetModuleHandle(NULL), szPath, MAX_PATH);
		int index = 0;
		CString sPath = szPath;
		for (int i=sPath.GetLength()-1; i>=0; i--)
			if (
				(sPath[i] == 92) &&
				(++index == 2)
			   )
			{
				sPath = sPath.Left(i) + _T("\\database.sqlite");
				break;
			}

	//  opening the database
		CString sError;
		if (!m_pDatabase->Connect(sPath, _T(""), _T(""), sError))
		{
			delete m_pDatabase;
			m_pDatabase = NULL;
			AfxMessageBox(_T("Failed to open database: " + sError));
			pDX->Fail();
		}

	//  loading employee names from the database
		CAbstractRecordset *pRecordset = NULL;
		if (!m_pDatabase->Query(_T("SELECT id, last_name FROM employee"), sError, &pRecordset))
		{
			delete m_pDatabase;
			m_pDatabase = NULL;
			AfxMessageBox(_T("Failed to open recordset: " + sError));
			pDX->Fail();
		}

	//  adding employee names to the list box
		while (pRecordset->Next())
		{
			int index = m_lstEmployee.AddString(pRecordset->AsString(1));  //  adding employee name to the list box
			m_lstEmployee.SetItemData(index, pRecordset->AsInteger(0));    //  remembering employee ID in the added list box item
		}

	//  closing the recordset and freing allocated resources
		pRecordset->Close();
		delete pRecordset;
		pRecordset = NULL;
	}
}

BEGIN_MESSAGE_MAP(CSampleDlg, CDialogEx)
	ON_BN_CLICKED(IDC_DELETE, &CSampleDlg::OnBnClickedDelete)
END_MESSAGE_MAP()

BOOL CSampleDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	SetIcon(m_hIcon, TRUE);
	SetIcon(m_hIcon, FALSE);

	return TRUE;
}

void CSampleDlg::OnBnClickedDelete()
{
//  checking if we have a record selected
	int index = m_lstEmployee.GetCurSel();
	if (LB_ERR == index)
	{
		AfxMessageBox(_T("Please select an item to delete"));
		return;
	}

//  constructing the query
	CString sQuery;
	int iEmployeeID = m_lstEmployee.GetItemData(index);
	sQuery.Format
		(
		_T("DELETE FROM employee WHERE id=%d"),
		iEmployeeID
		);

//  executing the query
	CString sError;
	if (!m_pDatabase->Query(sQuery, sError))
	{
		AfxMessageBox(_T("Failed to delete the record: ") + sError);
		return;
	}

//  deleting the item from the screen
	m_lstEmployee.DeleteString(index);
	AfxMessageBox(_T("Deleted successfully"));
}
